From 2682dee93ca4b7c780711a367a8c5b2d5ea94443 Mon Sep 17 00:00:00 2001
From: Shachaf Ben-Kiki <shachaf@gmail.com>
Date: Thu, 27 Dec 2012 12:15:16 -0800
Subject: [PATCH] Add Foldable and Traversable instances for (Either e), ((,)
 e), (Const e)

---
 Data/Foldable.hs    |   10 ++++++++++
 Data/Traversable.hs |   10 ++++++++++
 2 files changed, 20 insertions(+)

diff --git a/Data/Foldable.hs b/Data/Foldable.hs
index 0f23195..e4f755f 100644
--- a/Data/Foldable.hs
+++ b/Data/Foldable.hs
@@ -181,6 +181,16 @@ instance Foldable [] where
     foldr1 = Prelude.foldr1
     foldl1 = Prelude.foldl1
 
+instance Foldable (Either e) where
+    foldMap f (Right x) = f x
+    foldMap _ (Left _) = mempty
+
+instance Foldable ((,) e) where
+    foldMap f (_, x) = f x
+
+instance Foldable (Const m) where
+    foldMap _ _ = mempty
+
 instance Ix i => Foldable (Array i) where
     foldr f z = Prelude.foldr f z . elems
     foldl f z = Prelude.foldl f z . elems
diff --git a/Data/Traversable.hs b/Data/Traversable.hs
index 2ae49c6..4557db8 100644
--- a/Data/Traversable.hs
+++ b/Data/Traversable.hs
@@ -184,6 +184,16 @@ instance Traversable [] where
 
     mapM = Prelude.mapM
 
+instance Traversable (Either e) where
+    traverse f (Right x) = Right <$> f x
+    traverse _ (Left y) = pure (Left y)
+
+instance Traversable ((,) e) where
+    traverse f (x, y) = (,) x <$> f y
+
+instance Traversable (Const m) where
+    traverse f (Const m) = pure (Const m)
+
 instance Ix i => Traversable (Array i) where
     traverse f arr = listArray (bounds arr) `fmap` traverse f (elems arr)
 
-- 
1.7.10.4

